@@ -6,9 +6,9 @@ Framework of Information Theory for Electrophysiological data and Statistics
 """
 import logging
 
-from frites import io, core, stats, utils, workflow, simulations  # noqa
+from frites import io, core, conn, stats, utils, workflow, simulations  # noqa
 
-__version__ = "0.3.2"
+__version__ = "0.3.5"
 
 # -----------------------------------------------------------------------------
 # Set 'info' as the default logging level
