@@ -9,7 +9,7 @@ from frites.workflow.wf_base import WfBase
 from frites.workflow.wf_mi import WfMi
 from frites.workflow.wf_stats_ephy import WfStatsEphy
 
-from frites.core import it_fit
+from frites.conn import conn_fit
 from frites.io import (logger, set_log_level, convert_dfc_outputs)
 from frites.stats import ttest_1samp
 
@@ -89,10 +89,12 @@ class WfFit(WfBase):
         # get the number of pairs (source, target)
         n_roi = len(mi)
         if not net:
-            all_s, all_t = np.where(~np.eye(n_roi, dtype=bool))
+            all_s, all_t = dataset.get_connectivity_pairs(
+                nb_min_suj=dataset.nb_min_suj, directed=True)
             tail = 1
         else:
-            all_s, all_t = np.triu_indices(n_roi, k=1)
+            all_s, all_t = dataset.get_connectivity_pairs(
+                nb_min_suj=dataset.nb_min_suj, directed=False)
             tail = 0  # two tail test
         direction = 'bidirectional' if not net else 'unidirectional'
         logger.info(f"    Compute {direction} FIT (max_delay={max_delay}; "
@@ -136,7 +138,7 @@ class WfFit(WfBase):
 
     def fit(self, dataset, max_delay=0.3, net=False, mcp='cluster',
             cluster_th=None, cluster_alpha=0.05, n_perm=1000, n_jobs=-1,
-            random_state=None, output_type='3d_dataframe', **kw_stats):
+            random_state=None, **kw_stats):
         """Compute the Feature Specific Information transfer and statistics.
 
         In order to run the worflow, you must first provide a dataset instance
@@ -191,10 +193,6 @@ class WfFit(WfBase):
         random_state : int | None
             Fix the random state of the machine (use it for reproducibility).
             If None, a random state is randomly assigned.
-        output_type : string
-            Output format of the returned FIT and p-values. For details, see
-            :func:`frites.io.convert_dfc_outputs`. Use either '2d_array',
-            '3d_array', '2d_dataframe', '3d_dataframe', 'dataarray'.
         kw_stats : dict | {}
             Additional arguments to pass to the :class:`WfStatsEphy.fit`
             method
@@ -250,9 +248,9 @@ class WfFit(WfBase):
         # ---------------------------------------------------------------------
         # post-processing
         # ---------------------------------------------------------------------
-        logger.info(f"    Formatting output type ({output_type})")
+        logger.info(f"    Formatting output type")
         args = (self._times, dataset.roi_names, self._sources, self._targets,
-                output_type)
+                'dataarray')
         if isinstance(tvalues, np.ndarray):
             self._tvalues = convert_dfc_outputs(tvalues, *args)
         pvalues = convert_dfc_outputs(pvalues, is_pvalue=True, *args)
@@ -261,8 +259,8 @@ class WfFit(WfBase):
         elif inference is 'rfx':
             fit = np.stack(self._fit_m, axis=1)     # mean mi
         fit = convert_dfc_outputs(fit, *args)
-        if output_type is 'dataarray':
-            fit, pvalues = self._attrs_xarray(fit), self._attrs_xarray(pvalues)
+        # adding attributes
+        fit, pvalues = self._attrs_xarray(fit), self._attrs_xarray(pvalues)
 
         return fit, pvalues
 
@@ -353,13 +351,13 @@ def fcn_fit(x_s, x_t, xp_s, xp_t, suj_s, suj_t, times, max_delay, net,
             xp_s_suj.flags.writeable = True
             xp_t_suj.flags.writeable = True
             # FIT on true and permuted gcmi
-            _fit_suj = it_fit(x_s_suj, x_t_suj, times, max_delay)[0, ...]
-            _fitp_suj = it_fit(xp_s_suj, xp_t_suj, times, max_delay)
+            _fit_suj = conn_fit(x_s_suj, x_t_suj, times, max_delay)[0, ...]
+            _fitp_suj = conn_fit(xp_s_suj, xp_t_suj, times, max_delay)
             # compute unidirectional FIT
             if net:
                 # compute target -> source
-                _fit_ts = it_fit(x_t_suj, x_s_suj, times, max_delay)[0, ...]
-                _fitp_ts = it_fit(xp_t_suj, xp_s_suj, times, max_delay)
+                _fit_ts = conn_fit(x_t_suj, x_s_suj, times, max_delay)[0, ...]
+                _fitp_ts = conn_fit(xp_t_suj, xp_s_suj, times, max_delay)
                 # subtract to source -> target
                 _fit_suj -= _fit_ts
                 _fitp_suj -= _fitp_ts
